/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.j2ee.ejbverification;

import javax.lang.model.element.TypeElement;
import org.netbeans.api.java.source.CompilationInfo;
import org.netbeans.api.project.Project;
import org.netbeans.modules.j2ee.api.ejbjar.EjbJar;
import org.netbeans.modules.j2ee.dd.api.ejb.Ejb;
import org.openide.filesystems.FileObject;

/**
 *
 * @author Tomasz.Slota
 */
public class EJBProblemContext {

    private final FileObject fileObject;
    private final TypeElement clazz;
    private final Ejb ejb;
    private final SessionData sessionData;
    private final Project project;
    private final CompilationInfo complilationInfo;
    private final EjbJar ejbModule;

    public EJBProblemContext(
            CompilationInfo complilationInfo,
            Project project,
            EjbJar ejbModule,
            FileObject fileObject,
            TypeElement clazz,
            Ejb ejb,
            SessionData sessionData) {
        this.complilationInfo = complilationInfo;
        this.project = project;
        this.ejbModule = ejbModule;
        this.fileObject = fileObject;
        this.clazz = clazz;
        this.ejb = ejb;
        this.sessionData = sessionData;
    }
    
    public CompilationInfo getComplilationInfo() {
        return complilationInfo;
    }
    
    public FileObject getFileObject() {
        return fileObject;
    }

    public TypeElement getClazz() {
        return clazz;
    }

    public Ejb getEjb() {
        return ejb;
    }

    public SessionData getEjbData() {
        return sessionData;
    }

    public EjbJar getEjbModule() {
        return ejbModule;
    }

    public Project getProject() {
        return project;
    }

    /**
     * Stores all data necessary for the hints computation, to precompute most of information.
     */
    public static class SessionData {

        private final String[] businessLocal;
        private final String[] businessRemote;
        private final String sessionType;

        public SessionData(String[] businessLocal, String[] businessRemote, String sessionType) {
            this.businessLocal = businessLocal;
            this.businessRemote = businessRemote;
            this.sessionType = sessionType;
        }

        public String getSessionType() {
            return sessionType;
        }

        public String[] getBusinessLocal() {
            return businessLocal;
        }

        public String[] getBusinessRemote() {
            return businessRemote;
        }
    }
}
